# Copyright 2009-2014 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from requestbuilder import Arg, MutuallyExclusiveArgList
from requestbuilder.exceptions import ArgumentError

from euca2ools.commands.ec2 import EC2Request


class ModifyImageAttribute(EC2Request):
    DESCRIPTION = 'Modify an attribute of an image'
    ARGS = [Arg('ImageId', metavar='IMAGE', help='image to modify'),
            MutuallyExclusiveArgList(
                Arg('--description', dest='Description.Value', metavar='DESC',
                    help="change the image's description"),
                Arg('-p', '--product-code', dest='ProductCode', metavar='CODE',
                    action='append', help='''product code to add to the given
                    instance-store image'''),
                Arg('-l', '--launch-permission', action='store_true',
                    route_to=None,
                    help='grant/revoke launch permissions with -a/-r'))
            .required(),
            Arg('-a', '--add', metavar='ENTITY', action='append', default=[],
                route_to=None, help='''account to grant launch permission, or
                "all" for all accounts'''),
            Arg('-r', '--remove', metavar='ENTITY', action='append',
                default=[], route_to=None, help='''account to remove launch
                permission from, or "all" for all accounts''')]

    # noinspection PyExceptionInherit
    def preprocess(self):
        if self.args.get('launch_permission'):
            lperm = {}
            for entity in self.args.get('add', []):
                lperm.setdefault('Add', [])
                if entity == 'all':
                    lperm['Add'].append({'Group':  entity})
                else:
                    lperm['Add'].append({'UserId': entity})
            for entity in self.args.get('remove', []):
                lperm.setdefault('Remove', [])
                if entity == 'all':
                    lperm['Remove'].append({'Group':  entity})
                else:
                    lperm['Remove'].append({'UserId': entity})
            if not lperm:
                raise ArgumentError('at least one entity must be specified '
                                    'with -a/--add or -r/--remove')
            self.params['LaunchPermission'] = lperm
        else:
            if self.args.get('add'):
                raise ArgumentError('argument -a/--add may only be used '
                                    'with -l/--launch-permission')
            if self.args.get('remove'):
                raise ArgumentError('argument -r/--remove may only be used '
                                    'with -l/--launch-permission')

    def print_result(self, _):
        if self.args.get('Description.Value'):
            print self.tabify(('description', self.args['ImageId'],
                               None, self.args['Description.Value']))
        if self.args.get('ProductCode'):
            for code in self.args['ProductCode']:
                print self.tabify(('productcodes', self.args['ImageId'],
                                   'productCode', code))
        if self.args.get('launch_permission'):
            for add in self.params['LaunchPermission'].get('Add', []):
                for (entity_type, entity_name) in add.items():
                    print self.tabify(('launchPermission',
                                       self.args['ImageId'], 'ADD',
                                       entity_type, entity_name))
            for add in self.params['LaunchPermission'].get('Remove', []):
                for (entity_type, entity_name) in add.items():
                    print self.tabify(('launchPermission',
                                       self.args['ImageId'], 'REMOVE',
                                       entity_type, entity_name))



# Copyright 2009-2014 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import base64
from datetime import datetime, timedelta
import hashlib
import hmac
import json
import time

from requestbuilder import Arg
from requestbuilder.exceptions import ArgumentError

from euca2ools.commands.ec2 import EC2Request


class BundleInstance(EC2Request):
    DESCRIPTION = 'Bundle an S3-backed Windows instance'
    ARGS = [Arg('InstanceId', metavar='INSTANCE',
                help='ID of the instance to bundle (required)'),
            Arg('-b', '--bucket', dest='Storage.S3.Bucket', metavar='BUCKET',
                required=True, help='''bucket in which to store the new machine
                image (required)'''),
            Arg('-p', '--prefix', dest='Storage.S3.Prefix', metavar='PREFIX',
                required=True,
                help='beginning of the machine image bundle name (required)'),
            Arg('-o', '--owner-akid', '--user-access-key', metavar='KEY-ID',
                dest='Storage.S3.AWSAccessKeyId', help='''bucket owner's access
                key ID (required if not specified in configuration)'''),
            Arg('-c', '--policy', metavar='POLICY',
                dest='Storage.S3.UploadPolicy', help='''Base64-encoded upload
                policy that allows the server to upload a bundle on your
                behalf.  If unused, a secret key is required.'''),
            Arg('-s', '--policy-signature', metavar='SIGNATURE',
                dest='Storage.S3.UploadPolicySignature',
                help='''signature of the Base64-encoded upload policy.  If
                unused, a secret key is required.'''),
            Arg('-w', '--owner-sak', '--user-secret-key', metavar='KEY',
                route_to=None,
                help='''bucket owner's secret access key, used to sign upload
                policies.  This is required unless both -c and -s are used or
                if a secret key is specified in a configuration file.'''),
            Arg('-x', '--expires', metavar='HOURS', type=int, default=24,
                route_to=None,
                help='generated upload policy expiration time (default: 24)')]

    def generate_default_policy(self):
        delta = timedelta(hours=self.args['expires'])
        expire_time = (datetime.utcnow() + delta).replace(microsecond=0)

        conditions = [{'acl': 'ec2-bundle-read'},
                      {'bucket': self.args.get('Storage.S3.Bucket')},
                      ['starts-with', '$key',
                       self.args.get('Storage.S3.Prefix')]]
        policy = {'conditions': conditions,
                  'expiration': time.strftime('%Y-%m-%dT%H:%M:%SZ',
                                              expire_time.timetuple())}
        policy_json = json.dumps(policy)
        self.log.info('generated default policy: %s', policy_json)
        self.params['Storage.S3.UploadPolicy'] = base64.b64encode(policy_json)

    def sign_policy(self):
        my_hmac = hmac.new(self.args['owner_sak'], digestmod=hashlib.sha1)
        my_hmac.update(self.params.get('Storage.S3.UploadPolicy'))
        self.params['Storage.S3.UploadPolicySignature'] = \
            base64.b64encode(my_hmac.digest())

    # noinspection PyExceptionInherit
    def configure(self):
        EC2Request.configure(self)
        if not self.params.get('Storage.S3.AWSAccessKeyId'):
            config_key_id = self.config.get_user_option('key-id')
            if config_key_id:
                self.log.info('Using access key ID %s from configuration',
                              config_key_id)
                self.params['Storage.S3.AWSAccessKeyId'] = config_key_id
            else:
                raise ArgumentError('argument -o/--owner-akid is required')
        if not self.params.get('Storage.S3.UploadPolicy'):
            if not self.args.get('owner_sak'):
                config_secret_key = self.config.get_user_option('secret-key')
                if config_secret_key:
                    self.log.info('Using secret key from configuration')
                    self.args['owner_sak'] = config_secret_key
                else:
                    raise ArgumentError('argument -w/--owner-sak is required '
                                        'when -c/--policy is not used')
        elif not self.args.get('Storage.S3.UploadPolicySignature'):
            if not self.args.get('owner_sak'):
                config_secret_key = self.config.get_user_option('secret-key')
                if config_secret_key:
                    self.log.info('Using secret key from configuration')
                    self.args['owner_sak'] = config_secret_key
                else:
                    raise ArgumentError('argument -w/--owner-sak is required '
                                        'when -s/--policy-signature is not '
                                        'used')

    def preprocess(self):
        if not self.args.get('Storage.S3.UploadPolicy'):
            self.generate_default_policy()
        if not self.args.get('Storage.S3.UploadPolicySignature'):
            self.sign_policy()

    def print_result(self, result):
        self.print_bundle_task(result['bundleInstanceTask'])



# Copyright 2014 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import argparse
import base64
import sys

from requestbuilder import Arg, MutuallyExclusiveArgList
from requestbuilder.exceptions import ArgumentError

from euca2ools.commands.argtypes import b64encoded_file_contents
from euca2ools.commands.s3 import S3Request


class PostObject(S3Request):
    DESCRIPTION = ('Upload an object to the server using an upload policy\n\n'
                   'Note that uploading a large file to a region other than '
                   'the one the bucket is may result in "Broken pipe" errors '
                   'or other connection problems that this program cannot '
                   'detect.')
    AUTH_CLASS = None
    ARGS = [Arg('source', metavar='FILE', route_to=None,
                help='file to upload (required)'),
            Arg('dest', metavar='BUCKET/KEY', route_to=None,
                help='bucket and key name to upload the object to (required)'),
            MutuallyExclusiveArgList(
                Arg('--policy', dest='Policy', metavar='POLICY',
                    type=base64.b64encode,
                    help='upload policy to use for authorization'),
                Arg('--policy-file', dest='Policy', metavar='FILE',
                    type=b64encoded_file_contents, help='''file containing the
                    upload policy to use for authorization'''))
            .required(),
            Arg('--policy-signature', dest='Signature', required=True,
                help='signature for the upload policy (required)'),
            Arg('-I', '--access-key-id', dest='AWSAccessKeyId', required=True,
                metavar='KEY_ID', help='''ID of the access key that signed the
                upload policy (required)'''),
            # --security-token is an extension meant for eucalyptus's back end
            # to use for BundleInstance operations started by the web console.
            # https://eucalyptus.atlassian.net/browse/EUCA-9911
            # https://eucalyptus.atlassian.net/browse/TOOLS-511
            Arg('--security-token', dest='x-amz-security-token',
                default=argparse.SUPPRESS, help=argparse.SUPPRESS),
            Arg('--acl', default=argparse.SUPPRESS, choices=(
                'private', 'public-read', 'public-read-write',
                'authenticated-read', 'bucket-owner-read',
                'bucket-owner-full-control', 'aws-exec-read',
                'ec2-bundle-read'), help='''the ACL the object should have
                once uploaded.  Take care to ensure this satisfies any
                restrictions the upload policy may contain.'''),
            Arg('--mime-type', dest='Content-Type', default=argparse.SUPPRESS,
                help='MIME type for the file being uploaded')]
    METHOD = 'POST'

    # noinspection PyExceptionInherit
    def configure(self):
        S3Request.configure(self)

        if self.args['source'] == '-':
            self.files['file'] = sys.stdin
        elif isinstance(self.args['source'], basestring):
            self.files['file'] = open(self.args['source'])
        else:
            self.files['file'] = self.args['source']
        bucket, _, key = self.args['dest'].partition('/')
        if not bucket:
            raise ArgumentError('destination bucket name must be non-empty')
        if not key:
            raise ArgumentError('destination key name must be non-empty')

    # noinspection PyExceptionInherit
    def preprocess(self):
        # FIXME:  This should really stream the contents of the source rather
        # than reading it all into memory at once, but at the moment doing so
        # would require me to write a multipart MIME encoder that supports
        # both streaming and file rewinding.  Patches that do that are very
        # welcome.
        #
        # FIXME:  While you're in there, would you mind adding progress bar
        # support?  8^)
        self.path, _, self.params['key'] = self.args['dest'].partition('/')
        self.body = self.params
        self.params = None



# Copyright 2013 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from euca2ools.commands.elasticloadbalancing import ELBRequest
from requestbuilder import Arg
from requestbuilder.mixins import TabifyingMixin


class DescribeLoadBalancerPolicyTypes(ELBRequest, TabifyingMixin):
    DESCRIPTION = 'Show information about load balancer policy types'
    ARGS = [Arg('PolicyTypeNames.member', metavar='POLTYPE', nargs='*',
                help='limit results to specific policy types'),
            Arg('--show-long', action='store_true', route_to=None,
                help="show all of the policy types' info")]
    LIST_TAGS = ['PolicyTypeDescriptions', 'PolicyAttributeTypeDescriptions']

    def print_result(self, result):
        for poltype in result.get('PolicyTypeDescriptions', []):
            bits = ['POLICY_TYPE', poltype.get('PolicyTypeName'),
                    poltype.get('Description')]
            if self.args['show_long']:
                attrs = []
                for attr in poltype.get('PolicyAttributeTypeDescriptions', []):
                    elem_map = (('name', 'AttributeName'),
                                ('description', 'Description'),
                                ('type', 'AttributeType'),
                                ('default-value', 'DefaultValue'),
                                ('cardinality', 'Cardinality'))
                    attr_bits = []
                    for name, xmlname in elem_map:
                        attr_bits.append('='.join((name,
                                                   attr.get(xmlname) or '')))
                    attrs.append('{' + ','.join(attr_bits) + '}')
                bits.append(','.join(attrs))
            print self.tabify(bits)



# Copyright 2009-2014 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from euca2ools.commands.ec2 import EC2Request
from requestbuilder import Arg, Filter, GenericTagFilter
from requestbuilder.exceptions import ArgumentError


class DescribeImages(EC2Request):
    DESCRIPTION = ('Show information about images\n\nBy default, only images '
                   'your account owns and images for which your account has '
                   'explicit launch permissions are shown.')
    ARGS = [Arg('ImageId', metavar='IMAGE', nargs='*',
                help='limit results to specific images'),
            Arg('-a', '--all', action='store_true', route_to=None,
                help='describe all images'),
            Arg('-o', '--owner', dest='Owner', metavar='ACCOUNT',
                action='append',
                help='describe images owned by the specified owner'),
            Arg('-x', '--executable-by', dest='ExecutableBy',
                metavar='ACCOUNT', action='append',
                help='''describe images for which the specified account has
                explicit launch permissions''')]
    FILTERS = [Filter('architecture', help='CPU architecture'),
               Filter('block-device-mapping.delete-on-termination',
                      help='''whether a volume is deleted upon instance
                      termination'''),
               Filter('block-device-mapping.device-name',
                      help='device name for a volume mapped to the image'),
               Filter('block-device-mapping.snapshot-id',
                      help='snapshot ID for a volume mapped to the image'),
               Filter('block-device-mapping.volume-size',
                      help='volume size for a volume mapped to the image'),
               Filter('block-device-mapping.volume-type',
                      help='volume type for a volume mapped to the image'),
               Filter('description', help='image description'),
               Filter('hypervisor', help='image\'s hypervisor type'),
               Filter('image-id'),
               Filter('image-type',
                      help='image type ("machine", "kernel", or "ramdisk")'),
               Filter('is-public', help='whether the image is public'),
               Filter('kernel-id'),
               Filter('manifest-location'),
               Filter('name'),
               Filter('owner-alias', help="image owner's account alias"),
               Filter('owner-id', help="image owner's account ID"),
               Filter('platform', help='"windows" for Windows images'),
               Filter('product-code',
                      help='product code associated with the image'),
               Filter('product-code.type', help='''type of product code
                      associated with the image ("devpay", "marketplace")'''),
               Filter('ramdisk-id'),
               Filter('root-device-name'),
               Filter('root-device-type',
                      help='root device type ("ebs" or "instance-store")'),
               Filter('state', help='''image state ("available", "pending", or
                      "failed")'''),
               Filter('state-reason-code',
                      help='reason code for the most recent state change'),
               Filter('state-reason-message',
                      help='message for the most recent state change'),
               Filter('tag-key', help='key of a tag assigned to the image'),
               Filter('tag-value',
                      help='value of a tag assigned to the image'),
               GenericTagFilter('tag:KEY',
                                help='specific tag key/value combination'),
               Filter('virtualization-type',
                      help='virtualization type ("paravirtual" or "hvm")')]
    LIST_TAGS = ['imagesSet', 'productCodes', 'blockDeviceMapping', 'tagSet']

    # noinspection PyExceptionInherit
    def configure(self):
        EC2Request.configure(self)
        if self.args.get('all', False):
            if self.args.get('ImageId'):
                raise ArgumentError('argument -a/--all: not allowed with '
                                    'a list of images')
            if self.args.get('ExecutableBy'):
                raise ArgumentError('argument -a/--all: not allowed with '
                                    'argument -x/--executable-by')
            if self.args.get('Owner'):
                raise ArgumentError('argument -a/--all: not allowed with '
                                    'argument -o/--owner')

    def main(self):
        if not any(self.args.get(item) for item in ('all', 'ImageId',
                                                    'ExecutableBy', 'Owner')):
            # Default to owned images and images with explicit launch perms
            self.params['Owner'] = ['self']
            owned = self.send()
            del self.params['Owner']
            self.params['ExecutableBy'] = ['self']
            executable = self.send()
            del self.params['ExecutableBy']
            owned['imagesSet'] = (owned.get('imagesSet', []) +
                                  executable.get('imagesSet', []))
            return owned
        else:
            return self.send()

    def print_result(self, result):
        images = {}
        for image in result.get('imagesSet', []):
            images.setdefault(image['imageId'], image)
        for _, image in sorted(images.iteritems()):
            self.print_image(image)

    def print_image(self, image):
        if image.get('rootDeviceType') == 'instance-store':
            imagename = image.get('imageLocation')
        else:
            imagename = '/'.join((image.get('imageOwnerId', ''),
                                  image.get('name')))

        print self.tabify((
            'IMAGE', image.get('imageId'), imagename,
            image.get('imageOwnerAlias') or image.get('imageOwnerId'),
            image.get('imageState'),
            ('public' if image.get('isPublic') == 'true' else 'private'),
            image.get('architecture'), image.get('imageType'),
            image.get('kernelId'), image.get('ramdiskId'),
            image.get('platform'), image.get('rootDeviceType'),
            image.get('virtualizationType'), image.get('hypervisor')))
        for mapping in image.get('blockDeviceMapping', []):
            self.print_blockdevice_mapping(mapping)
        for tag in image.get('tagSet', []):
            self.print_resource_tag(tag, image.get('imageId'))



# Copyright 2013 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from euca2ools.commands.autoscaling import AutoScalingRequest
from requestbuilder import Arg


class PutScalingPolicy(AutoScalingRequest):
    DESCRIPTION = "Create or update an auto-scaling group's scaling policy"
    ARGS = [Arg('PolicyName', metavar='POLICY',
                help='name of the policy to create or update (required)'),
            Arg('-g', '--auto-scaling-group', dest='AutoScalingGroupName',
                metavar='ASGROUP', required=True,
                help='''name of the auto-scaling group the policy is associated
                with (required)'''),
            Arg('-a', '--adjustment', dest='ScalingAdjustment',
                metavar='SCALE', type=int, required=True,
                help='''amount to scale the group's capacity of the group.  Use
                a negative value, as in "--adjustment=-1", to decrease
                capacity. (required)'''),
            Arg('-t', '--type', dest='AdjustmentType', required=True,
                choices=('ChangeInCapacity', 'ExactCapacity',
                         'PercentChangeInCapacity'), help='''whether the
                adjustment is the new desired size or an increment to the
                group's current capacity. An increment can either be a fixed
                number or a percentage of current capacity.  (required)'''),
            Arg('--cooldown', dest='Cooldown', metavar='SECONDS', type=int,
                help='''waiting period after successful auto-scaling activities
                during which later auto-scaling activities will not
                execute'''),
            Arg('-s', '--min-adjustment-step', dest='MinAdjustmentStep',
                type=int, metavar='PERCENT',
                help='''for a PercentChangeInCapacity type policy, guarantee
                that this policy will change the group's desired capacity by at
                least this much''')]

    def print_result(self, result):
        print result.get('PolicyARN')



# Copyright 2013-2014 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from requestbuilder import Arg, Filter, GenericTagFilter

from euca2ools.commands.ec2 import EC2Request


class DescribeNetworkAcls(EC2Request):
    DESCRIPTION = 'Describe one or more network ACLs'
    ARGS = [Arg('NetworkAclId', metavar='NACL', nargs='*',
                help='limit results to one or more network ACLs')]
    FILTERS = [Filter('association.association-id',
                      help='ID of an association ID for a network ACL'),
               Filter('association.network-acl-id', help='''ID of the
                      network ACL involved in an association'''),
               Filter('association.subnet-id',
                      help='ID of the subnet involved in an association'),
               Filter('default', choices=('true', 'false'), help='''whether
                      the network ACL is the default for its VPC'''),
               Filter('entry.cidr', help='CIDR range for a network ACL entry'),
               Filter('entry.egress', choices=('true', 'false'),
                      help='whether an entry applies to egress traffic'),
               Filter('entry.icmp.code', type=int,
                      help='ICMP code for a network ACL entry'),
               Filter('entry.icmp.type', type=int,
                      help='ICMP type for a network ACL entry'),
               Filter('entry.port-range.from', type=int,
                      help='start of the port range for a network ACL entry'),
               Filter('entry.port-range.to', type=int,
                      help='end of the port range for a network ACL entry'),
               Filter('entry.protocol',
                      help='protocol for a network ACL entry'),
               Filter('entry.rule-action', choices=('allow', 'deny'), help='''
                      whether a network ACL entry allows or denies traffic'''),
               Filter('entry.rule-number', type=int,
                      help='rule number of a network ACL entry'),
               Filter('network-acl-id'),
               Filter('tag-key',
                      help='key of a tag assigned to the network ACL'),
               Filter('tag-value',
                      help='value of a tag assigned to the network ACL'),
               GenericTagFilter('tag:KEY',
                                help='specific tag key/value combination'),
               Filter('vpc-id', help="the VPC's ID")]

    LIST_TAGS = ['associationSet', 'entrySet', 'networkAclSet', 'tagSet']

    def print_result(self, result):
        for acl in result.get('networkAclSet') or []:
            self.print_network_acl(acl)



# Copyright 2009-2014 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from euca2ools.commands.ec2 import EC2Request
from requestbuilder import Arg, Filter, GenericTagFilter


class DescribeInstances(EC2Request):
    DESCRIPTION = 'Show information about instances'
    ARGS = [Arg('InstanceId', metavar='INSTANCE', nargs='*',
                help='limit results to specific instances')]
    FILTERS = [Filter('architecture', help='CPU architecture'),
               Filter('association.allocation-id',
                      help='''[VPC only] allocation ID bound to a network
                      interface's elastic IP address'''),
               Filter('association.association-id', help='''[VPC only]
                      association ID returned when an elastic IP was associated
                      with a network interface'''),
               Filter('association.ip-owner-id',
                      help='''[VPC only] ID of the owner of the elastic IP
                      address associated with a network interface'''),
               Filter('association.public-ip', help='''[VPC only] address of
                      the elastic IP address bound to a network interface'''),
               Filter('availability-zone'),
               Filter('block-device-mapping.attach-time',
                      help='volume attachment time'),
               Filter('block-device-mapping.delete-on-termination',
                      help='''whether a volume is deleted upon instance
                      termination'''),
               Filter('block-device-mapping.device-name',
                      help='volume device name (e.g. /dev/sdf)'),
               Filter('block-device-mapping.status', help='volume status'),
               Filter('block-device-mapping.volume-id', help='volume ID'),
               Filter('client-token',
                      help='idempotency token provided at instance run time'),
               Filter('dns-name', help='public DNS name'),
               # EC2's documentation for "group-id" refers VPC users to
               # "instance.group-id", while their documentation for the latter
               # refers them to the former.  Consequently, I'm not going to
               # document a difference for either.  They both seem to work for
               # non-VPC instances.
               Filter('group-id', help='security group ID'),
               Filter('group-name', help='security group name'),
               Filter('hypervisor', help='hypervisor type'),
               Filter('iam-instance-profile.arn',
                      help="ARN of the instance's instance profile"),
               Filter('image-id', help='machine image ID'),
               Filter('instance.group-id', help='security group ID'),
               Filter('instance.group-name', help='security group name'),
               Filter('instance-id'),
               Filter('instance-lifecycle',
                      help='whether this is a spot instance'),
               Filter('instance-state-code', type=int,
                      help='numeric code identifying instance state'),
               Filter('instance-state-name', help='instance state'),
               Filter('instance-type'),
               Filter('ip-address', help='public IP address'),
               Filter('kernel-id', help='kernel image ID'),
               Filter('key-name',
                      help='key pair name provided at instance launch time'),
               Filter('launch-index',
                      help='launch index within a reservation'),
               Filter('launch-time', help='instance launch time'),
               Filter('monitoring-state',
                      help='monitoring state ("enabled" or "disabled")'),
               Filter('network-interface.addresses.association.ip-owner-id',
                      help='''[VPC only] ID of the owner of the private IP
                      address associated with a network interface'''),
               Filter('network-interface.addresses.association.public-ip',
                      help='''[VPC only] ID of the association of an elastic IP
                      address with a network interface'''),
               Filter('network-interface.addresses.primary',
                      help='''[VPC only] whether the IP address of the VPC
                      network interface is the primary private IP address
                      ("true" or "false")'''),
               Filter('network-interface.addresses.private-ip-address',
                      help='''[VPC only] network interface's private IP
                      address'''),
               Filter('network-interface.attachment.device-index', type=int,
                      help='''[VPC only] device index to which a network
                      interface is attached'''),
               Filter('network-interface.attachment.attach-time',
                      help='''[VPC only] time a network interface was attached
                      to an instance'''),
               Filter('network-interface.attachment.attachment-id',
                      help='''[VPC only] ID of a network interface's
                      attachment'''),
               Filter('network-interface.attachment.delete-on-termination',
                      help='''[VPC only] whether a network interface attachment
                      is deleted when an instance is terminated ("true" or
                      "false")'''),
               Filter('network-interface.attachment.instance-owner-id',
                      help='''[VPC only] ID of the instance to which a network
                      interface is attached'''),
               Filter('network-interface.attachment.status',
                      help="[VPC only] network interface's attachment status"),
               Filter('network-interface.availability-zone',
                      help="[VPC only] network interface's availability zone"),
               Filter('network-interface.description',
                      help='[VPC only] description of a network interface'),
               Filter('network-interface.group-id',
                      help="[VPC only] network interface's security group ID"),
               Filter('network-interface.group-name', help='''[VPC only]
                      network interface's security group name'''),
               Filter('network-interface.mac-address',
                      help="[VPC only] network interface's hardware address"),
               Filter('network-interface.network-interface.id',
                      help='[VPC only] ID of a network interface'),
               Filter('network-interface.owner-id',
                      help="[VPC only] ID of a network interface's owner"),
               Filter('network-interface.private-dns-name',
                      help="[VPC only] network interface's private DNS name"),
               Filter('network-interface.requester-id',
                      help="[VPC only] network interface's requester ID"),
               Filter('network-interface.requester-managed',
                      help='''[VPC only] whether the network interface is
                      managed by the service'''),
               Filter('network-interface.source-destination-check',
                      help='''[VPC only] whether source/destination checking is
                      enabled for a network interface ("true" or "false")'''),
               Filter('network-interface.status',
                      help="[VPC only] network interface's status"),
               Filter('network-interface.subnet-id',
                      help="[VPC only] ID of a network interface's subnet"),
               Filter('network-interface.vpc-id',
                      help="[VPC only] ID of a network interface's VPC"),
               Filter('owner-id', help="instance owner's account ID"),
               Filter('placement-group-name'),
               Filter('platform', help='"windows" for Windows instances'),
               Filter('private-dns-name'),
               Filter('private-ip-address'),
               Filter('product-code'),
               Filter('product-code.type',
                      help='type of product code ("devpay" or "marketplace")'),
               Filter('ramdisk-id', help='ramdisk image ID'),
               Filter('reason',
                      help="reason for the instance's current state"),
               Filter('requester-id',
                      help='ID of the entity that launched an instance'),
               Filter('reservation-id',
                      help='''ID of the instance's reservation'''),
               Filter('root-device-name',
                      help='root device name (e.g. /dev/sda1)'),
               Filter('root-device-type',
                      help='root device type ("ebs" or "instance-store")'),
               Filter('spot-instance-request-id'),
               Filter('state-reason-code',
                      help='reason code for the most recent state change'),
               Filter('state-reason-message',
                      help='message describing the most recent state change'),
               Filter('subnet-id',
                      help='[VPC only] ID of the subnet the instance is in'),
               Filter('tag-key',
                      help='name of any tag assigned to the instance'),
               Filter('tag-value',
                      help='value of any tag assigned to the instance'),
               GenericTagFilter('tag:KEY',
                                help='specific tag key/value combination'),
               Filter('virtualization-type'),
               Filter('vpc-id',
                      help='[VPC only] ID of the VPC the instance is in')]
    LIST_TAGS = ['reservationSet', 'instancesSet', 'groupSet', 'tagSet',
                 'blockDeviceMapping', 'productCodes', 'networkInterfaceSet',
                 'privateIpAddressesSet']

    def print_result(self, result):
        for reservation in result.get('reservationSet'):
            self.print_reservation(reservation)



# Copyright 2009-2015 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from euca2ools.commands.ec2 import EC2Request
from requestbuilder import Arg, Filter, GenericTagFilter
from requestbuilder.exceptions import ArgumentError


class DescribeSnapshots(EC2Request):
    DESCRIPTION = ('Show information about snapshots\n\nBy default, only '
                   'snapshots your account owns and snapshots for which your '
                   'account has explicit restore permissions are shown.')
    ARGS = [Arg('SnapshotId', nargs='*', metavar='SNAPSHOT',
                help='limit results to specific snapshots'),
            Arg('-a', '--all', action='store_true', route_to=None,
                help='describe all snapshots'),
            Arg('-o', '--owner', dest='Owner', metavar='ACCOUNT',
                action='append', default=[],
                help='limit results to snapshots owned by specific accounts'),
            Arg('-r', '--restorable-by', dest='RestorableBy', action='append',
                metavar='ACCOUNT', default=[], help='''limit results to
                snapahots restorable by specific accounts''')]
    FILTERS = [Filter('description', help='snapshot description'),
               Filter('owner-alias', help="snapshot owner's account alias"),
               Filter('owner-id', help="snapshot owner's account ID"),
               Filter('progress', help='snapshot progress, in percentage'),
               Filter('snapshot-id'),
               Filter('start-time', help='snapshot initiation time'),
               Filter('status'),
               Filter('tag-key', help='key of a tag assigned to the snapshot'),
               Filter('tag-value',
                      help='value of a tag assigned to the snapshot'),
               GenericTagFilter('tag:KEY',
                                help='specific tag key/value combination'),
               Filter('volume-id', help='source volume ID'),
               Filter('volume-size', type=int)]
    LIST_TAGS = ['snapshotSet', 'tagSet']

    # noinspection PyExceptionInherit
    def configure(self):
        EC2Request.configure(self)
        if self.args.get('all'):
            if self.args.get('Owner'):
                raise ArgumentError('argument -a/--all: not allowed with '
                                    'argument -o/--owner')
            if self.args.get('RestorableBy'):
                raise ArgumentError('argument -a/--all: not allowed with '
                                    'argument -r/--restorable-by')

    def main(self):
        if not any(self.args.get(item) for item in ('all', 'SnapshotId',
                                                    'Owner', 'RestorableBy')):
            # Default to owned snapshots and those with explicit restore perms
            self.params['Owner'] = ['self']
            owned = self.send()
            del self.params['Owner']
            self.params['RestorableBy'] = ['self']
            restorable = self.send()
            del self.params['RestorableBy']
            owned['snapshotSet'] = (owned.get('snapshotSet', []) +
                                    restorable.get('snapshotSet', []))
            return owned
        else:
            return self.send()

    def print_result(self, result):
        for snapshot in result.get('snapshotSet', []):
            self.print_snapshot(snapshot)



# Copyright 2015 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import os
import pipes
import sys

from requestbuilder import Arg, MutuallyExclusiveArgList
from requestbuilder.command import BaseCommand

from euca2ools.commands import Euca2ools


class ReleaseRole(BaseCommand):
    DESCRIPTION = '''\
        Release IAM role credentials

        The %(prog)s utility removes the credentials created by
        euare-assumerole(1) by outputting shellcode that deletes
        the environment variables it creates.  Use it inside an eval
        command to make this process seamless:

            $ eval `euare-releaserole`

        Note that if the credentials used to initially assume the role
        were supplied in the form of environment variables those
        environment variables will need to be reset:

            $ source eucarc'''
    SUITE = Euca2ools
    ARGS = [MutuallyExclusiveArgList(
                Arg('-c', dest='csh_output', route_to=None,
                    action='store_true', help='''generate C-shell commands on
                    stdout (default if SHELL looks like a csh-style shell'''),
                Arg('-s', dest='sh_output', route_to=None,
                    action='store_true', help='''generate Bourne shell
                    commands on stdout (default if SHELL does not look
                    like a csh-style shell'''))]

    def print_result(self, _):
        for var in (
                'AWS_ACCESS_KEY_ID', 'AWS_ACCESS_KEY', 'EC2_ACCESS_KEY',
                'AWS_SECRET_ACCESS_KEY', 'AWS_SECRET_KEY',
                'EC2_SECRET_KEY', 'AWS_SESSION_TOKEN',
                'AWS_SECURITY_TOKEN', 'AWS_CREDENTIAL_EXPIRATION',
                'EC2_USER_ID', 'AWS_CREDENTIAL_FILE'):
            if (self.args.get('csh_output') or
                    (not self.args.get('sh_output') and
                     os.getenv('SHELL', '').endswith('csh'))):
                fmt = 'unsetenv {0};'
            else:
                fmt = 'unset {0};'
            print fmt.format(var)
        print
        print '# If you can read this, rerun this program with eval:'
        print '#     eval `{0}`'.format(
            ' '.join(pipes.quote(arg) for arg in sys.argv))



# Copyright (c) 2009-2016 Hewlett Packard Enterprise Development LP
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import argparse
import multiprocessing
import os.path

from requestbuilder import Arg
from requestbuilder.mixins import FileTransferProgressBarMixin

from euca2ools.bundle.manifest import BundleManifest
from euca2ools.commands.bundle.mixins import BundleUploadingMixin
from euca2ools.commands.s3 import S3Request
from euca2ools.commands.s3.putobject import PutObject


class UploadBundle(S3Request, BundleUploadingMixin,
                   FileTransferProgressBarMixin):
    DESCRIPTION = 'Upload a bundle prepared by euca-bundle-image to the cloud'
    ARGS = [Arg('-m', '--manifest', metavar='FILE', required=True,
                help='manifest for the bundle to upload (required)'),
            Arg('-d', '--directory', metavar='DIR',
                help='''directory that contains the bundle parts (default:
                directory that contains the manifest)'''),
            # TODO:  make this work
            Arg('--part', metavar='INT', type=int, default=0,
                help=argparse.SUPPRESS),
            Arg('--skipmanifest', action='store_true',
                help='do not upload the manifest')]

    def configure(self):
        # This goes before configure because -S's absence causes
        # self.auth.configure to blow up.  With an upload policy that
        # is undesirable.
        self.configure_bundle_upload_auth()
        S3Request.configure(self)

    def main(self):
        key_prefix = self.get_bundle_key_prefix()
        self.ensure_dest_bucket_exists()

        manifest = BundleManifest.read_from_file(self.args['manifest'])
        part_dir = (self.args.get('directory') or
                    os.path.dirname(self.args['manifest']))
        for part in manifest.image_parts:
            part.filename = os.path.join(part_dir, part.filename)
            if not os.path.isfile(part.filename):
                raise ValueError("no such part: '{0}'".format(part.filename))

        # manifest -> upload
        part_out_r, part_out_w = multiprocessing.Pipe(duplex=False)
        part_gen = multiprocessing.Process(target=_generate_bundle_parts,
                                           args=(manifest, part_out_w))
        part_gen.start()
        part_out_w.close()

        # Drive the upload process by feeding in part info
        self.upload_bundle_parts(part_out_r, key_prefix,
                                 show_progress=self.args.get('show_progress'))
        part_gen.join()

        # (conditionally) upload the manifest
        if not self.args.get('skipmanifest'):
            manifest_dest = (key_prefix +
                             os.path.basename(self.args['manifest']))
            req = PutObject.from_other(
                self, source=self.args['manifest'], dest=manifest_dest,
                acl=self.args.get('acl') or 'aws-exec-read',
                retries=self.args.get('retries') or 0)
            req.main()
        else:
            manifest_dest = None

        return {'parts': tuple({'filename': part.filename,
                                'key': (key_prefix +
                                        os.path.basename(part.filename))}
                               for part in manifest.image_parts),
                'manifests': ({'filename': self.args['manifest'],
                               'key': manifest_dest},)}

    def print_result(self, result):
        if self.debug:
            for part in result['parts']:
                print 'Uploaded', part['key']
        if result['manifests'][0]['key'] is not None:
            print 'Uploaded', result['manifests'][0]['key']


def _generate_bundle_parts(manifest, out_mpconn):
    try:
        for part in manifest.image_parts:
            assert os.path.isfile(part.filename)
            out_mpconn.send(part)
    finally:
        out_mpconn.close()



# Copyright 2013-2014 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import argparse
import hashlib
import socket
import sys
import threading
import time

from requestbuilder import Arg
from requestbuilder.exceptions import (ArgumentError, ClientError,
                                       TimeoutError)
from requestbuilder.mixins import FileTransferProgressBarMixin

from euca2ools.commands.s3 import S3Request
import euca2ools.util


class PutObject(S3Request, FileTransferProgressBarMixin):
    DESCRIPTION = ('Upload an object to the server\n\nNote that uploading a '
                   'large file to a region other than the one the bucket is '
                   'may result in "Broken pipe" errors or other connection '
                   'problems that this program cannot detect.')
    ARGS = [Arg('source', metavar='FILE', route_to=None,
                help='file to upload (required)'),
            Arg('dest', metavar='BUCKET/KEY', route_to=None,
                help='bucket and key name to upload the object to (required)'),
            Arg('--size', type=int, route_to=None, help='''the number of
                bytes to upload (required when reading from stdin)'''),
            Arg('--acl', route_to=None, choices=(
                'private', 'public-read', 'public-read-write',
                'authenticated-read', 'bucket-owner-read',
                'bucket-owner-full-control', 'aws-exec-read')),
            Arg('--mime-type', route_to=None,
                help='MIME type for the file being uploaded'),
            Arg('--retry', dest='retries', action='store_const', const=5,
                default=0, route_to=None,
                help='retry interrupted uploads up to 5 times'),
            Arg('--progressbar-label', help=argparse.SUPPRESS)]
    METHOD = 'PUT'

    def __init__(self, **kwargs):
        S3Request.__init__(self, **kwargs)
        self.last_upload_error = None
        self._lock = threading.Lock()

    # noinspection PyExceptionInherit
    def configure(self):
        S3Request.configure(self)
        if self.args['source'] == '-':
            if self.args.get('size') is None:
                raise ArgumentError(
                    "argument --size is required when uploading stdin")
            source = _FileObjectExtent(sys.stdin, self.args['size'])
        elif isinstance(self.args['source'], basestring):
            source = _FileObjectExtent.from_filename(
                self.args['source'], size=self.args.get('size'))
        else:
            if self.args.get('size') is None:
                raise ArgumentError(
                    "argument --size is required when uploading a file object")
            source = _FileObjectExtent(self.args['source'], self.args['size'])
        self.args['source'] = source
        bucket, _, key = self.args['dest'].partition('/')
        if not bucket:
            raise ArgumentError('destination bucket name must be non-empty')
        if not key:
            raise ArgumentError('destination key name must be non-empty')

    def preprocess(self):
        self.path = self.args['dest']
        if self.args.get('acl'):
            self.headers['x-amz-acl'] = self.args['acl']
        if self.args.get('mime_type'):
            self.headers['Content-Type'] = self.args['mime_type']

    # noinspection PyExceptionInherit
    def main(self):
        self.preprocess()
        source = self.args['source']
        self.headers['Content-Length'] = source.size

        # We do the upload in another thread so the main thread can show a
        # progress bar.
        upload_thread = threading.Thread(
            target=self.try_send, args=(source,),
            kwargs={'retries_left': self.args.get('retries') or 0})
        # The upload thread is daemonic so ^C will kill the program more
        # cleanly.
        upload_thread.daemon = True
        upload_thread.start()
        pbar_label = self.args.get('progressbar_label') or source.filename
        pbar = self.get_progressbar(label=pbar_label, maxval=source.size)
        pbar.start()
        while upload_thread.is_alive():
            pbar.update(source.tell())
            time.sleep(0.05)
        pbar.finish()
        upload_thread.join()
        source.close()
        with self._lock:
            if self.last_upload_error is not None:
                # pylint: disable=E0702
                raise self.last_upload_error
                # pylint: enable=E0702

    def try_send(self, source, retries_left=0):
        self.body = source
        if retries_left > 0 and not source.can_rewind:
            self.log.warn('source cannot rewind, so requested retries will '
                          'not be attempted')
            retries_left = 0
        try:
            response = self.send()
            our_md5 = source.read_hexdigest
            their_md5 = response.headers['ETag'].lower().strip('"')
            if their_md5 != our_md5:
                self.log.error('corrupt upload (our MD5: %s, their MD5: %s',
                               our_md5, their_md5)
                raise ClientError('upload was corrupted during transit')
        except ClientError as err:
            if isinstance(err, TimeoutError):
                if retries_left > 0:
                    self.log.info('retrying upload (%i retry attempt(s) '
                                  'remaining)', retries_left)
                    source.rewind()
                    return self.try_send(source, retries_left - 1)
            with self._lock:
                self.log.error('upload failed', exc_info=True)
                self.last_upload_error = err
            return
        except Exception as err:
            with self._lock:
                self.log.error('upload failed', exc_info=True)
                self.last_upload_error = err
            return


class _FileObjectExtent(object):
    # By rights this class should be iterable, but if we do that then requests
    # will attempt to use chunked transfer-encoding, which S3 does not
    # support.

    def __init__(self, fileobj, size, filename=None):
        self.closed = False
        self.filename = filename
        self.fileobj = fileobj
        self.size = size
        self.__bytes_read = 0
        self.__md5 = hashlib.md5()
        if hasattr(self.fileobj, 'tell'):
            self.__initial_pos = self.fileobj.tell()
        else:
            self.__initial_pos = None

    def __len__(self):
        return self.size

    @classmethod
    def from_filename(cls, filename, size=None):
        if size is None:
            size = euca2ools.util.get_filesize(filename)
        return cls(open(filename), size, filename=filename)

    @property
    def can_rewind(self):
        return hasattr(self.fileobj, 'seek') and self.__initial_pos is not None

    def close(self):
        self.fileobj.close()
        self.closed = True

    def next(self):
        remaining = self.size - self.__bytes_read
        if remaining <= 0:
            raise StopIteration()
        chunk = self.fileobj.next()  # might raise StopIteration, which is good
        chunk = chunk[:remaining]  # throw away data that are off the end
        self.__bytes_read += len(chunk)
        self.__md5.update(chunk)
        return chunk

    def read(self, size=-1):
        remaining = self.size - self.__bytes_read
        if size < 0:
            chunk_len = remaining
        else:
            chunk_len = min(remaining, size)
        chunk = self.fileobj.read(chunk_len)
        self.__bytes_read += len(chunk)
        self.__md5.update(chunk)
        return chunk

    @property
    def read_hexdigest(self):
        return self.__md5.hexdigest()

    def rewind(self):
        if not hasattr(self.fileobj, 'seek'):
            raise TypeError('file object is not seekable')
        assert self.__initial_pos is not None
        self.fileobj.seek(self.__initial_pos)
        self.__bytes_read = 0
        self.__md5 = hashlib.md5()

    def tell(self):
        return self.__bytes_read



# Copyright 2013-2014 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from requestbuilder import Arg
from requestbuilder.exceptions import ArgumentError
from requestbuilder.mixins import TabifyingMixin

from euca2ools.commands.ec2 import EC2Request


class ModifyInstanceTypeAttribute(EC2Request, TabifyingMixin):
    DESCRIPTION = '[Eucalyptus cloud admin only] Modify an instance type'
    ARGS = [Arg('Name', metavar='INSTANCETYPE',
                help='name of the instance type to modify (required)'),
            Arg('-c', '--cpus', dest='Cpu', metavar='COUNT', type=int,
                help='number of virtual CPUs to allocate to each instance'),
            Arg('-d', '--disk', dest='Disk', metavar='GiB', type=int,
                help='amount of instance storage to allow each instance'),
            Arg('-m', '--memory', dest='Memory', metavar='MiB', type=int,
                help='amount of RAM to allocate to each instance'),
            Arg('--reset', dest='Reset', action='store_true',
                help='reset the instance type to its default configuration')]

    # noinspection PyExceptionInherit
    def configure(self):
        EC2Request.configure(self)
        if (self.args.get('Reset') and
                any(self.args.get(attr) is not None for attr in
                    ('Cpu', 'Disk', 'Memory'))):
            # Basically, reset is mutually exclusive with everything else.
            raise ArgumentError('argument --reset may not be used with '
                                'instance type attributes')

    def print_result(self, result):
        newtype = result.get('instanceType', {})
        print self.tabify(('INSTANCETYPE', newtype.get('name'),
                           newtype.get('cpu'), newtype.get('memory'),
                           newtype.get('disk')))



# Copyright 2009-2015 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import argparse

from requestbuilder import Arg

from euca2ools.commands.iam import IAMRequest, AS_ACCOUNT, arg_user
from euca2ools.commands.iam.deleteaccesskey import DeleteAccessKey
from euca2ools.commands.iam.deleteloginprofile import DeleteLoginProfile
from euca2ools.commands.iam.deletesigningcertificate import \
    DeleteSigningCertificate
from euca2ools.commands.iam.deleteuserpolicy import DeleteUserPolicy
from euca2ools.commands.iam.getloginprofile import GetLoginProfile
from euca2ools.commands.iam.listaccesskeys import ListAccessKeys
from euca2ools.commands.iam.listgroupsforuser import ListGroupsForUser
from euca2ools.commands.iam.listsigningcertificates import \
    ListSigningCertificates
from euca2ools.commands.iam.listuserpolicies import ListUserPolicies
from euca2ools.commands.iam.removeuserfromgroup import RemoveUserFromGroup
from euca2ools.exceptions import AWSError


class DeleteUser(IAMRequest):
    DESCRIPTION = 'Delete a user'
    ARGS = [arg_user(help='name of the user to delete (required)'),
            Arg('-r', '--recursive', action='store_true', route_to=None,
                help='''remove all IAM resources associated with the user
                        first'''),
            Arg('-R', '--recursive-euca', dest='IsRecursive',
                action='store_const', const='true', help=argparse.SUPPRESS),
            Arg('-p', '--pretend', action='store_true', route_to=None,
                help='''list the resources that would be deleted instead of
                        actually deleting them. Implies -r.'''),
            AS_ACCOUNT]

    def main(self):
        if self.args['recursive'] or self.args['pretend']:
            # Figure out what we'd have to delete
            req = ListAccessKeys.from_other(
                self, UserName=self.args['UserName'],
                DelegateAccount=self.params['DelegateAccount'])
            keys = req.main().get('AccessKeyMetadata', [])
            req = ListUserPolicies.from_other(
                self, UserName=self.args['UserName'],
                DelegateAccount=self.params['DelegateAccount'])
            policies = req.main().get('PolicyNames', [])
            req = ListSigningCertificates.from_other(
                self, UserName=self.args['UserName'],
                DelegateAccount=self.params['DelegateAccount'])
            certs = req.main().get('Certificates', [])
            req = ListGroupsForUser.from_other(
                self, UserName=self.args['UserName'],
                DelegateAccount=self.params['DelegateAccount'])
            groups = req.main().get('Groups', [])
            req = GetLoginProfile.from_other(
                self, UserName=self.args['UserName'],
                DelegateAccount=self.params['DelegateAccount'])
            try:
                # This will raise an exception if no login profile is found.
                req.main()
                has_login_profile = True
            except AWSError as err:
                if err.code == 'NoSuchEntity':
                    # It doesn't exist
                    has_login_profile = False
                else:
                    # Something else went wrong; not our problem
                    raise
        else:
            # Just in case
            keys = []
            policies = []
            certs = []
            groups = []
            has_login_profile = False
        if self.args['pretend']:
            return {'keys': keys, 'policies': policies,
                    'certificates': certs, 'groups': groups,
                    'has_login_profile': has_login_profile}
        else:
            if self.args['recursive']:
                for key in keys:
                    req = DeleteAccessKey.from_other(
                        self, UserName=self.args['UserName'],
                        AccessKeyId=key['AccessKeyId'],
                        DelegateAccount=self.params['DelegateAccount'])
                    req.main()
                for policy in policies:
                    req = DeleteUserPolicy.from_other(
                        self, UserName=self.args['UserName'],
                        PolicyName=policy,
                        DelegateAccount=self.params['DelegateAccount'])
                    req.main()
                for cert in certs:
                    req = DeleteSigningCertificate.from_other(
                        self, UserName=self.args['UserName'],
                        CertificateId=cert['CertificateId'],
                        DelegateAccount=self.params['DelegateAccount'])
                    req.main()
                for group in groups:
                    req = RemoveUserFromGroup.from_other(
                        self, user_names=[self.args['UserName']],
                        GroupName=group['GroupName'],
                        DelegateAccount=self.params['DelegateAccount'])
                    req.main()
                if has_login_profile:
                    req = DeleteLoginProfile.from_other(
                        self, UserName=self.args['UserName'],
                        DelegateAccount=self.params['DelegateAccount'])
                    req.main()
            return self.send()

    def print_result(self, result):
        if self.args['pretend']:
            print 'accesskeys'
            for key in result['keys']:
                print '\t' + key['AccessKeyId']
            print 'policies'
            for policy in result['policies']:
                print '\t' + policy
            print 'certificates'
            for cert in result['certificates']:
                print '\t' + cert['CertificateId']
            print 'groups'
            for group in result['groups']:
                print '\t' + group['Arn']



# Copyright 2015 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from requestbuilder import Arg, Filter

from euca2ools.commands.bootstrap import BootstrapRequest


class DescribeServiceCertificates(BootstrapRequest):
    DESCRIPTION = 'Show information about one or more service certificates'
    ARGS = [Arg('--format', dest='Format', choices=('pem', 'der'),
                type=str.lower, help='format to output certificates in'),
            Arg('--digest', dest='FingerprintDigest', type=str.upper,
                choices=('SHA-1', 'SHA-256'),
                help='digest algorithm to use for certificate fingerprints')]
    FILTERS = [Filter('certificate-usage'),
               Filter('service-type')]
    LIST_TAGS = ['serviceCertificates']



# Copyright 2015 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import os

from requestbuilder import Arg
from requestbuilder.auth.aws import HmacKeyAuth
from requestbuilder.command import BaseCommand
import requestbuilder.exceptions
from requestbuilder.mixins import RegionConfigurableMixin

from euca2ools.commands import Euca2ools
import euca2ools.util


class GenerateEnvironment(BaseCommand, RegionConfigurableMixin):
    DESCRIPTION = ('Read environment variables and euca2ools.ini(5) '
                   'files to discover the service URLs and credentials '
                   'for a region, then output shellcode with the '
                   'corresponding environment variables for that '
                   'information.  This output will contain secret '
                   'access keys and should be treated with care.')
    SUITE = Euca2ools
    ARGS = [Arg('--simple', action='store_true', help='''use a simpler
                output format intended for consumption by scripts'''),
            HmacKeyAuth.ARGS]

    def configure(self):
        BaseCommand.configure(self)
        self.update_config_view()

    def main(self):
        env_vars = {}
        services = euca2ools.util.generate_service_names()
        for service, service_var in services.items():
            url = os.getenv(service_var)
            if not url:
                url = self.config.get_region_option('{0}-url'.format(service))
            env_vars[service_var] = url
        auth = HmacKeyAuth(config=self.config, loglevel=self.log.level,
                           **self.args)
        try:
            auth.configure()
        except requestbuilder.exceptions.AuthError:
            self.log.info('auth configuration failed; info may be missing',
                          exc_info=True)
        env_vars['AWS_ACCESS_KEY_ID'] = auth.args.get('key_id')
        env_vars['AWS_SECRET_ACCESS_KEY'] = auth.args.get('secret_key')
        env_vars['AWS_SECURITY_TOKEN'] = auth.args.get('security_token')
        env_vars['AWS_CREDENTIAL_EXPIRATION'] = auth.args.get(
            'credential_expiration')
        env_vars['EC2_USER_ID'] = os.getenv(
            'EC2_USER_ID', self.config.get_user_option('account-id'))
        env_vars['EC2_CERT'] = os.getenv(
            'EC2_CERT', self.config.get_user_option('certificate'))
        env_vars['EC2_PRIVATE_KEY'] = os.getenv(
            'EC2_PRIVATE_KEY', self.config.get_user_option('private-key'))
        env_vars['EUCALYPTUS_CERT'] = os.getenv(
            'EUCALYPTUS_CERT', self.config.get_region_option('certificate'))
        return env_vars

    def print_result(self, env_vars):
        if self.args.get('simple'):
            for key, val in sorted(env_vars.items()):
                print '{key}={val}'.format(key=key, val=(val or ''))
        else:
            # We do this in two steps so we can put all the comments last,
            # allowing the output to work inside a shell's eval statement.
            for key, val in sorted(env_vars.items()):
                if val:
                    print '{key}={val}; export {key};'.format(key=key, val=val)
            for key, val in sorted(env_vars.items()):
                if not val:
                    print '# {key} is not set'.format(key=key)



# Copyright 2013 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import argparse

from requestbuilder import Arg, MutuallyExclusiveArgList
from requestbuilder.mixins import TabifyingMixin

from euca2ools.commands.autoscaling import AutoScalingRequest


class TerminateInstanceInAutoScalingGroup(AutoScalingRequest,
                                          TabifyingMixin):
    DESCRIPTION = "Manually terminate an auto-scaling instance"
    ARGS = [Arg('InstanceId', metavar='INSTANCE',
                help='ID of the instance to terminate (required)'),
            MutuallyExclusiveArgList(
                Arg('-d', '--decrement-desired-capacity', action='store_const',
                    dest='ShouldDecrementDesiredCapacity', const='true',
                    help='''also reduce the desired capacity of the
                    auto-scaling group by 1'''),
                Arg('-D', '--no-decrement-desired-capacity',
                    dest='ShouldDecrementDesiredCapacity',
                    action='store_const', const='false',
                    help='''leave the auto-scaling group's desired capacity
                    as-is.  A new instance may be launched to compensate for
                    the one being terminated.'''))
            .required(),
            Arg('--show-long', action='store_true', route_to=None,
                help='show extra info about the instance being terminated'),
            Arg('-f', '--force', action='store_true', route_to=None,
                help=argparse.SUPPRESS)]  # for compatibility

    def print_result(self, result):
        activity = result['Activity']
        bits = ['INSTANCE',
                activity.get('ActivityId'),
                activity.get('EndTime'),
                activity.get('StatusCode'),
                activity.get('Cause')]
        if self.args['show_long']:
            bits.append(activity.get('StatusMessage'))
            bits.append(activity.get('Progress'))
            bits.append(activity.get('Description'))
            bits.append(activity.get('StartTime'))
        print self.tabify(bits)



# Copyright 2013 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import argparse
from euca2ools.commands.autoscaling import AutoScalingRequest
from requestbuilder import Arg
from requestbuilder.mixins import TabifyingMixin
from requestbuilder.response import PaginatedResponse


class DescribeAutoScalingInstances(AutoScalingRequest, TabifyingMixin):
    DESCRIPTION = 'Describe instances in auto-scaling groups'
    ARGS = [Arg('InstanceIds.member', metavar='INSTANCE', nargs='*',
                help='limit results to specific instances'),
            Arg('--show-long', action='store_true', route_to=None,
                help=argparse.SUPPRESS)]  # Often typed out of habit
    LIST_TAGS = ['AutoScalingInstances']

    def main(self):
        return PaginatedResponse(self, (None,), ('AutoScalingInstances',))

    def prepare_for_page(self, page):
        # Pages are defined by NextToken
        self.params['NextToken'] = page

    def get_next_page(self, response):
        return response.get('NextToken') or None

    def print_result(self, result):
        for instance in result.get('AutoScalingInstances', []):
            print self.tabify(('INSTANCE',
                               instance.get('InstanceId'),
                               instance.get('AutoScalingGroupName'),
                               instance.get('AvailabilityZone'),
                               instance.get('LifecycleState'),
                               instance.get('HealthStatus'),
                               instance.get('LaunchConfigurationName')))



# Copyright 2009-2015 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import datetime
import getpass
import inspect
import os.path
import pkgutil
import stat
import struct
import sys
import tempfile

import requestbuilder.service

import euca2ools.commands


def build_progressbar_label_template(fnames):
    if len(fnames) == 0:
        return None
    elif len(fnames) == 1:
        return '{fname}'
    else:
        max_fname_len = max(len(os.path.basename(fname)) for fname in fnames)
        fmt_template = '{{fname:<{maxlen}}} ({{index:>{lenlen}}}/{total})'
        return fmt_template.format(maxlen=max_fname_len,
                                   lenlen=len(str(len(fnames))),
                                   total=len(fnames))


# pylint: disable=W0622
def mkdtemp_for_large_files(suffix='', prefix='tmp', dir=None):
    """
    Like tempfile.mkdtemp, but using /var/tmp as a last resort instead of /tmp.

    This is meant for utilities that create large files, as /tmp is often a
    ramdisk.
    """

    if dir is None:
        dir = (os.getenv('TMPDIR') or os.getenv('TEMP') or os.getenv('TMP') or
               '/var/tmp')
    return tempfile.mkdtemp(suffix=suffix, prefix=prefix, dir=dir)
# pylint: enable=W0622


def prompt_for_password():
    pass1 = getpass.getpass(prompt='New password: ')
    pass2 = getpass.getpass(prompt='Retype new password: ')
    if pass1 == pass2:
        return pass1
    else:
        print >> sys.stderr, 'error: passwords do not match'
        return prompt_for_password()


def strip_response_metadata(response_dict):
    useful_keys = [key for key in response_dict if key != 'ResponseMetadata']
    if len(useful_keys) == 1:
        return response_dict[useful_keys[0]] or {}
    else:
        return response_dict


def build_iam_policy(effect, resources, actions):
    policy = {'Statement': []}
    for resource in resources or []:
        sid = datetime.datetime.utcnow().strftime('Stmt%Y%m%d%H%M%S%f')
        statement = {'Sid': sid, 'Effect': effect, 'Action': actions,
                     'Resource': resource}
        policy['Statement'].append(statement)
    return policy


def get_filesize(filename):
    mode = os.stat(filename).st_mode
    if stat.S_ISBLK(mode):
        # os.path.getsize doesn't work on block devices, but we can use lseek
        # to figure it out
        block_fd = os.open(filename, os.O_RDONLY)
        try:
            return os.lseek(block_fd, 0, os.SEEK_END)
        finally:
            os.close(block_fd)
    elif any((stat.S_ISCHR(mode), stat.S_ISFIFO(mode), stat.S_ISSOCK(mode),
              stat.S_ISDIR(mode))):
        raise TypeError("'{0}' does not have a usable file size"
                        .format(filename))
    return os.path.getsize(filename)


def get_vmdk_image_size(filename):
    if get_filesize(filename) < 1024:
        raise ValueError('File {0} is to small to be a valid Stream'
                         ' Optimized VMDK'.format(filename))
    # see https://www.vmware.com/support/developer/vddk/vmdk_50_technote.pdf
    # for header/footer format
    with open(filename, 'rb') as disk:
        data = struct.unpack('<iiiqqqqiqqq?bbbbh433c', disk.read(512))
        if data[9] & 0xffffffffffffffff == 0:
            # move to 1024 bytes from the end and read footer
            disk.seek(-1024, 2)
            data = struct.unpack('<iiiqqqqiqqq?bbbbh433c', disk.read(512))

    # validate
    if 1447904331 != data[0]:
        raise ValueError('File {0} is not a Stream Optimized VMDK'
                         .format(filename))
    if data[2] & 0x10000 == 0:
        raise ValueError('File {0} does not contain compressed parts'
                         .format(filename))
    if data[2] & 0x20000 == 0:
        raise ValueError('File {0} does not have all data present'
                         .format(filename))
    if data[11]:
        raise ValueError('File {0} marked with unclean shutdown'
                         .format(filename))
    if data[16] != 1:
        raise ValueError('File {0} uses unsupported compression algorithm'
                         .format(filename))

    return 512 * data[3]


def check_dict_whitelist(dict_, err_context, whitelist=None):
    if not isinstance(dict_, dict):
        raise ValueError('{0} must be a dict'.format(err_context))
    if whitelist:
        differences = set(dict_.keys()) - set(whitelist)
        if differences:
            raise ValueError('unrecognized {0} argument(s): {1}'
                             .format(err_context, ', '.join(differences)))


def transform_dict(dict_, transformation_dict):
    transformed = {}
    for key, val in dict_.iteritems():
        if key in transformation_dict:
            transformed[transformation_dict[key]] = val
        else:
            transformed[key] = val
    return transformed


def add_fake_region_name(service):
    """
    If no name for a region is otherwise defined (i.e. service.region_name
    is None and the AWS_AUTH_REGION environment variable is not set),
    log a warning and add a fake region name so HmacV4Auth has something
    to work with.  This works because eucalyptus doesn't care what name
    one chooses for a region.

    Setups that use eucarc files against AWS will still be broken.

    This was added in euca2ools the 3.3 series and should be removed
    some time after that.
    """

    if service.region_name is None and not os.getenv('AWS_AUTH_REGION'):
        service.region_name = 'undefined-{0}'.format(os.getpid())
        service.log.warn('added fake region name %s', service.region_name)


def generate_service_names():
    """
    Generate a dict with keys for each service and values for those
    services' corresponding URL environment variables, if any.
    """
    services = {'properties': 'EUCA_PROPERTIES_URL',
                'reporting': 'EUCA_REPORTING_URL'}
    for importer, modname, ispkg in pkgutil.iter_modules(
            euca2ools.commands.__path__, euca2ools.commands.__name__ + '.'):
        module = __import__(modname, fromlist='dummy')
        for name, obj in inspect.getmembers(module):
            if (inspect.isclass(obj) and inspect.getmodule(obj) == module and
                    issubclass(obj, requestbuilder.service.BaseService)):
                services[obj.NAME] = obj.URL_ENVVAR
    return services



# Copyright 2015 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from requestbuilder import Arg
import requestbuilder.auth.aws
import requestbuilder.request
import requestbuilder.service

from euca2ools.commands import Euca2ools
from euca2ools.exceptions import AWSError
from euca2ools.util import add_fake_region_name


class Bootstrap(requestbuilder.service.BaseService):
    # Also known as Empyrean
    NAME = 'bootstrap'
    DESCRIPTION = '[Eucalyptus only] Bootstrap service'
    API_VERSION = 'eucalyptus'
    REGION_ENVVAR = 'AWS_DEFAULT_REGION'
    URL_ENVVAR = 'EUCA_BOOTSTRAP_URL'

    ARGS = [Arg('-U', '--url', metavar='URL',
                help='[Eucalyptus only] bootstrap service endpoint URL')]

    def configure(self):
        requestbuilder.service.BaseService.configure(self)
        add_fake_region_name(self)

    def handle_http_error(self, response):
        raise AWSError(response)


class BootstrapRequest(requestbuilder.request.AWSQueryRequest):
    SUITE = Euca2ools
    SERVICE_CLASS = Bootstrap
    AUTH_CLASS = requestbuilder.auth.aws.HmacV4Auth
    METHOD = 'POST'



# Copyright (c) 2009-2016 Hewlett Packard Enterprise Development LP
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from euca2ools.commands.ec2 import EC2Request
from requestbuilder import Arg, Filter, GenericTagFilter


class DescribeSecurityGroups(EC2Request):
    DESCRIPTION = ('Show information about security groups\n\nNote that '
                   'filters are matched on literal strings only, so '
                   '"--filter ip-permission.from-port=22" will *not* match a '
                   'group with a port range of 20 to 30.')
    ARGS = [Arg('group', metavar='GROUP', nargs='*', route_to=None,
                default=[], help='limit results to specific security groups')]
    FILTERS = [Filter('description', help='group description'),
               Filter('group-id'),
               Filter('group-name'),
               Filter('ip-permission.cidr',
                      help='CIDR IP range granted permission by the group'),
               Filter('ip-permission.from-port',
                      help='start of TCP/UDP port range, or ICMP type number'),
               Filter('ip-permission.group-name', help='''name of another group
                      granted permission by this group'''),
               Filter('ip-permission.group-id', help='''ID of another group
                      granted permission by this group'''),
               Filter('ip-permission.protocol',
                      help='IP protocol for the permission'),
               Filter('ip-permission.to-port',
                      help='end of TCP/UDP port range, or ICMP code'),
               Filter('ip-permission.user-id',
                      help='ID of an account granted permission'),
               Filter('owner-id', help="account ID of the group's owner"),
               Filter('tag-key', help='key of a tag assigned to the group'),
               Filter('tag-value',
                      help='value of a tag assigned to the group'),
               GenericTagFilter('tag:KEY',
                                help='specific tag key/value combination'),
               Filter('vpc-id',
                      help='[VPC only] ID of a VPC the group belongs to')]
    LIST_TAGS = ['securityGroupInfo', 'ipPermissions', 'ipPermissionsEgress',
                 'groups', 'ipRanges', 'tagSet']

    def preprocess(self):
        for group in self.args['group']:
            if group.startswith('sg-'):
                self.params.setdefault('GroupId', [])
                self.params['GroupId'].append(group)
            else:
                self.params.setdefault('GroupName', [])
                self.params['GroupName'].append(group)

    def print_result(self, result):
        for group in result.get('securityGroupInfo', []):
            self.print_group(group)

    def print_group(self, group):
        print self.tabify(('GROUP', group.get('groupId'), group.get('ownerId'),
                           group.get('groupName'),
                           group.get('groupDescription'),
                           group.get('vpcId')))
        for perm in group.get('ipPermissions', []):
            perm_base = ['PERMISSION', group.get('ownerId'),
                         group.get('groupName'), 'ALLOWS',
                         perm.get('ipProtocol'), perm.get('fromPort'),
                         perm.get('toPort')]
            for cidr_range in perm.get('ipRanges', []):
                perm_item = ['FROM', 'CIDR', cidr_range.get('cidrIp'),
                             'ingress']
                print self.tabify(perm_base + perm_item)
            for othergroup in perm.get('groups', []):
                perm_item = ['FROM', 'USER', othergroup.get('userId')]
                if othergroup.get('groupName'):
                    perm_item.extend(['NAME', othergroup['groupName']])
                if othergroup.get('groupId'):
                    perm_item.extend(['ID', othergroup['groupId']])
                perm_item.append('ingress')
                print self.tabify(perm_base + perm_item)
        for perm in group.get('ipPermissionsEgress', []):
            perm_base = ['PERMISSION', group.get('ownerId'),
                         group.get('groupName'), 'ALLOWS',
                         perm.get('ipProtocol'), perm.get('fromPort'),
                         perm.get('toPort')]
            for cidr_range in perm.get('ipRanges', []):
                perm_item = ['TO', 'CIDR', cidr_range.get('cidrIp'), 'egress']
                print self.tabify(perm_base + perm_item)
            for othergroup in perm.get('groups', []):
                perm_item = ['TO', 'USER', othergroup.get('userId')]
                if othergroup.get('groupName'):
                    perm_item.extend(['NAME', othergroup['groupName']])
                if othergroup.get('groupId'):
                    perm_item.extend(['ID', othergroup['groupId']])
                perm_item.append('egress')
                print self.tabify(perm_base + perm_item)
        for tag in group.get('tagSet', []):
            self.print_resource_tag(tag, (group.get('groupId') or
                                          group.get('groupName')))



# Copyright 2014 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from requestbuilder import Arg

from euca2ools.commands.cloudformation import CloudFormationRequest


class DescribeStacks(CloudFormationRequest):
    DESCRIPTION = 'Describe one or more stacks'
    ARGS = [Arg('StackName', metavar='STACK', nargs="?",
                help='limit results to a single stack'),
            Arg('--show-long', action='store_true', route_to=None,
                help="show all of the stacks' info")]
    LIST_TAGS = ['Outputs', 'Parameters', 'Stacks']

    def print_result(self, result):
        for stack in result['Stacks']:
            self.print_stack(stack)
            if self.args['show_long']:
                print stack.get('StackId')
                print stack.get('NotificationARNs')
            if stack.get('Parameters'):
                for param in stack.get('Parameters'):
                    self.print_parameter(param)
            if stack.get('Outputs'):
                for output in stack.get('Outputs'):
                    self.print_output(output)



# Copyright 2013-2014 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import binascii
import hashlib
import logging
import os.path
import subprocess

import lxml.etree
import lxml.objectify

import euca2ools.bundle


class BundleManifest(object):
    def __init__(self, loglevel=None):
        self.log = logging.getLogger(self.__class__.__name__)
        if loglevel is not None:
            self.log.level = loglevel
        self.image_arch = None
        self.kernel_id = None
        self.ramdisk_id = None
        self.block_device_mappings = {}  # virtual -> device
        self.product_codes = []
        self.image_name = None
        self.account_id = None
        self.image_type = None
        self.image_digest = None
        self.image_digest_algorithm = None
        self.image_size = None
        self.bundled_image_size = None
        self.enc_key = None
        self.enc_iv = None
        self.enc_algorithm = None
        self.image_parts = []

    @classmethod
    def read_from_file(cls, manifest_filename, privkey_filename=None):
        with open(manifest_filename) as manifest_fileobj:
            return cls.read_from_fileobj(manifest_fileobj, privkey_filename)

    @classmethod
    def read_from_fileobj(cls, manifest_fileobj, privkey_filename=None):
        xml = lxml.objectify.parse(manifest_fileobj).getroot()
        manifest = cls()
        mconfig = xml.machine_configuration
        manifest.image_arch = mconfig.architecture.text.strip()
        if hasattr(mconfig, 'kernel_id'):
            manifest.kernel_id = mconfig.kernel_id.text.strip()
        if hasattr(mconfig, 'ramdisk_id'):
            manifest.ramdisk_id = mconfig.ramdisk_id.text.strip()
        if hasattr(mconfig, 'block_device_mappings'):
            for xml_mapping in mconfig.block_device_mappings.iter(
                    tag='block_device_mapping'):
                device = xml_mapping.device.text.strip()
                virtual = xml_mapping.virtual.text.strip()
                manifest.block_device_mappings[virtual] = device
        if hasattr(mconfig, 'productcodes'):
            for xml_pcode in mconfig.productcodes.iter(tag='product_code'):
                manifest.product_codes.append(xml_pcode.text.strip())
        manifest.image_name = xml.image.name.text.strip()
        manifest.account_id = xml.image.user.text.strip()
        manifest.image_type = xml.image.type.text.strip()
        manifest.image_digest = xml.image.digest.text.strip()
        manifest.image_digest_algorithm = xml.image.digest.get('algorithm')
        manifest.image_size = int(xml.image.size.text.strip())
        manifest.bundled_image_size = int(xml.image.bundled_size.text.strip())
        if privkey_filename is not None:
            try:
                manifest.enc_key = _decrypt_hex(
                    xml.image.user_encrypted_key.text.strip(),
                    privkey_filename)
            except (AttributeError, ValueError):
                manifest.enc_key = _decrypt_hex(
                    xml.image.ec2_encrypted_key.text.strip(), privkey_filename)
            manifest.enc_algorithm = xml.image.user_encrypted_key.get(
                'algorithm')
            try:
                manifest.enc_iv = _decrypt_hex(
                    xml.image.user_encrypted_iv.text.strip(), privkey_filename)
            except (AttributeError, ValueError):
                manifest.enc_iv = _decrypt_hex(
                    xml.image.ec2_encrypted_iv.text.strip(), privkey_filename)

        manifest.image_parts = [None] * int(xml.image.parts.get('count'))
        for xml_part in xml.image.parts.iter(tag='part'):
            index = int(xml_part.get('index'))
            manifest.image_parts[index] = euca2ools.bundle.BundlePart(
                xml_part.filename.text.strip(), xml_part.digest.text.strip(),
                xml_part.digest.get('algorithm'))
        for index, part in enumerate(manifest.image_parts):
            if part is None:
                raise ValueError('part {0} must not be None'.format(index))
        return manifest

    def dump_to_str(self, privkey_filename, user_cert_filename,
                    ec2_cert_filename, pretty_print=False):
        if self.enc_key is None:
            raise ValueError('enc_key must not be None')
        if self.enc_iv is None:
            raise ValueError('enc_iv must not be None')
        ec2_fp = euca2ools.bundle.util.get_cert_fingerprint(ec2_cert_filename)
        self.log.info('creating manifest for EC2 service with fingerprint %s',
                      ec2_fp)
        self.log.debug('EC2 certificate:  %s', ec2_cert_filename)
        self.log.debug('user certificate: %s', user_cert_filename)
        self.log.debug('user private key: %s', privkey_filename)

        xml = lxml.objectify.Element('manifest')

        # Manifest version
        xml.version = '2007-10-10'

        # Our version
        xml.bundler = None
        xml.bundler.name = 'euca2ools'
        xml.bundler.version = euca2ools.__version__
        xml.bundler.release = 0

        # Target hardware
        xml.machine_configuration = None
        mconfig = xml.machine_configuration
        assert self.image_arch is not None
        mconfig.architecture = self.image_arch
        if self.image_type == 'machine':
            if self.block_device_mappings:
                mconfig.block_device_mapping = None
                for virtual, device in sorted(
                        self.block_device_mappings.items()):
                    xml_mapping = lxml.objectify.Element('mapping')
                    xml_mapping.virtual = virtual
                    xml_mapping.device = device
                    mconfig.block_device_mapping.append(xml_mapping)
            if self.product_codes:
                mconfig.product_codes = None
                for code in self.product_codes:
                    xml_code = lxml.objectify.Element('product_code')
                    mconfig.product_codes.append(xml_code)
                    mconfig.product_codes.product_code[-1] = code
        # kernel_id and ramdisk_id are normally meaningful only for machine
        # images, but eucalyptus also uses them to indicate kernel and ramdisk
        # images using the magic string "true", so their presence cannot be
        # made contingent on whether the image is a machine image or not.  Be
        # careful not to create invalid kernel or ramdisk manifests because of
        # this.
        if self.kernel_id:
            mconfig.kernel_id = self.kernel_id
        if self.ramdisk_id:
            mconfig.ramdisk_id = self.ramdisk_id

        # Image info
        xml.image = None
        assert self.image_name is not None
        xml.image.name = self.image_name
        assert self.account_id is not None
        xml.image.user = self.account_id

        # xml.image.type must appear immediately after xml.image.user
        # for EC2 compatibility.
        assert self.image_type is not None
        xml.image.type = self.image_type

        assert self.image_digest is not None
        xml.image.digest = self.image_digest
        assert self.image_digest_algorithm is not None
        xml.image.digest.set('algorithm', self.image_digest_algorithm)

        assert self.image_size is not None
        xml.image.size = self.image_size
        assert self.bundled_image_size is not None
        xml.image.bundled_size = self.bundled_image_size

        # Bundle encryption keys (these are cloud-specific)
        assert self.enc_key is not None
        assert self.enc_iv is not None
        assert self.enc_algorithm is not None
        xml.image.ec2_encrypted_key = _public_encrypt(self.enc_key,
                                                      ec2_cert_filename)
        xml.image.ec2_encrypted_key.set('algorithm', self.enc_algorithm)
        if user_cert_filename:
            xml.image.user_encrypted_key = _public_encrypt(self.enc_key,
                                                           user_cert_filename)
        else:
            # Absence results in 400 (InvalidManifest)
            xml.image.user_encrypted_key = None
        xml.image.user_encrypted_key.set('algorithm', self.enc_algorithm)
        xml.image.ec2_encrypted_iv = _public_encrypt(self.enc_iv,
                                                     ec2_cert_filename)
        if user_cert_filename:
            xml.image.user_encrypted_iv = _public_encrypt(self.enc_iv,
                                                          user_cert_filename)
        else:
            # Absence results in 400 (InvalidManifest)
            xml.image.user_encrypted_iv = None

        # Bundle parts
        xml.image.parts = None
        xml.image.parts.set('count', str(len(self.image_parts)))
        for index, part in enumerate(self.image_parts):
            if part is None:
                raise ValueError('part {0} must not be None'.format(index))
            part_elem = lxml.objectify.Element('part')
            part_elem.set('index', str(index))
            part_elem.filename = os.path.basename(part.filename)
            part_elem.digest = part.hexdigest
            part_elem.digest.set('algorithm', part.digest_algorithm)
            # part_elem.append(lxml.etree.Comment(
            #     ' size: {0} '.format(part.size)))
            xml.image.parts.append(part_elem)

        # Cleanup for signature
        lxml.objectify.deannotate(xml, xsi_nil=True)
        lxml.etree.cleanup_namespaces(xml)
        if privkey_filename:
            to_sign = (lxml.etree.tostring(xml.machine_configuration) +
                       lxml.etree.tostring(xml.image))
            signature = _rsa_sha1_sign(to_sign, privkey_filename)
        else:
            # Absence yields 400 (InvalidManifest)
            # Empty contents yield 500 (InternalError)
            signature = 'UNSIGNED'
        xml.signature = signature
        self.log.debug('hex-encoded signature: %s', signature)
        lxml.objectify.deannotate(xml, xsi_nil=True)
        lxml.etree.cleanup_namespaces(xml)
        self.log.debug('-- manifest content --\n', extra={'append': True})
        pretty_manifest = lxml.etree.tostring(xml, pretty_print=True).strip()
        self.log.debug('%s', pretty_manifest, extra={'append': True})
        self.log.debug('-- end of manifest content --')
        return lxml.etree.tostring(xml, xml_declaration=True,
                                   pretty_print=pretty_print).strip()

    def dump_to_file(self, manifest_file, privkey_filename,
                     user_cert_filename, ec2_cert_filename,
                     pretty_print=False):
        manifest_file.write(self.dump_to_str(
            privkey_filename, user_cert_filename, ec2_cert_filename,
            pretty_print=pretty_print))


def _decrypt_hex(hex_encrypted_key, privkey_filename):
    popen = subprocess.Popen(['openssl', 'rsautl', '-decrypt', '-pkcs',
                              '-inkey', privkey_filename],
                             stdin=subprocess.PIPE, stdout=subprocess.PIPE)
    binary_encrypted_key = binascii.unhexlify(hex_encrypted_key)
    (decrypted_key, _) = popen.communicate(binary_encrypted_key)
    try:
        # Make sure it might actually be an encryption key.
        # This isn't perfect, but it's still better than nothing.
        int(decrypted_key, 16)
        return decrypted_key
    except ValueError:
        pass
    raise ValueError("Failed to decrypt the bundle's encryption key.  "
                     "Ensure the key supplied matches the one used for "
                     "bundling.")


def _public_encrypt(content, cert_filename):
    popen = subprocess.Popen(['openssl', 'rsautl', '-encrypt', '-pkcs',
                              '-inkey', cert_filename, '-certin'],
                             stdin=subprocess.PIPE, stdout=subprocess.PIPE)
    (stdout, _) = popen.communicate(content)
    return binascii.hexlify(stdout)


def _rsa_sha1_sign(content, privkey_filename):
    digest = hashlib.sha1()
    digest.update(content)
    popen = subprocess.Popen(['openssl', 'pkeyutl', '-sign', '-inkey',
                              privkey_filename, '-pkeyopt', 'digest:sha1'],
                             stdin=subprocess.PIPE, stdout=subprocess.PIPE)
    (stdout, _) = popen.communicate(digest.digest())
    return binascii.hexlify(stdout)



# Copyright 2013 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from euca2ools.commands.monitoring import CloudWatchRequest
from requestbuilder import Arg
from requestbuilder.mixins import TabifyingMixin
from requestbuilder.response import PaginatedResponse


class DescribeAlarms(CloudWatchRequest, TabifyingMixin):
    DESCRIPTION = 'Describe alarms'
    ARGS = [Arg('AlarmNames.member', metavar='ALARM', nargs='*',
                help='limit results to specific alarms'),
            Arg('--action-prefix', dest='ActionPrefix', metavar='PREFIX',
                help='''limit results to alarms whose actions' ARNs begin with
                a specific string'''),
            Arg('--alarm-name-prefix', dest='AlarmNamePrefix',
                metavar='PREFIX', help='''limit results to alarms whose names
                begin with a specific string'''),
            Arg('--show-long', action='store_true', route_to=None,
                help="show all of the alarms' info"),
            Arg('--state-value', dest='StateValue',
                choices=('OK', 'ALARM', 'INSUFFICIENT_DATA'),
                help='limit results to alarms in a specific state')]
    LIST_TAGS = ['MetricAlarms', 'AlarmActions', 'Dimensions',
                 'InsufficientDataActions', 'OKActions']

    def main(self):
        return PaginatedResponse(self, (None,), ('MetricAlarms',))

    def prepare_for_page(self, page):
        self.params['NextToken'] = page

    def get_next_page(self, response):
        return response.get('NextToken') or None

    def print_result(self, result):
        for alarm in result.get('MetricAlarms', []):
            self.print_alarm(alarm)



# Copyright 2009-2015 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from distutils.command.build_py import build_py
from distutils.command.build_scripts import build_scripts
from distutils.command.install_scripts import install_scripts
from distutils.command.sdist import sdist
import glob
import os.path
import sys

from setuptools import find_packages, setup

from euca2ools import __version__


REQUIREMENTS = ['lxml',
                'PyYAML',
                'requestbuilder>=0.4',
                'requests',
                'six>=1.4']
if sys.version_info < (2, 7):
    REQUIREMENTS.append('argparse')


# Cheap hack:  install symlinks separately from regular files.
# cmd.copy_tree accepts a preserve_symlinks option, but when we call
# ``setup.py install'' more than once the method fails when it encounters
# symlinks that are already there.

class build_scripts_except_symlinks(build_scripts):
    '''Like build_scripts, but ignoring symlinks'''
    def copy_scripts(self):
        orig_scripts = self.scripts
        self.scripts = [script for script in self.scripts
                        if not os.path.islink(script)]
        build_scripts.copy_scripts(self)
        self.scripts = orig_scripts


class install_scripts_and_symlinks(install_scripts):
    '''Like install_scripts, but also replicating nonexistent symlinks'''
    def run(self):
        install_scripts.run(self)
        # Replicate symlinks if they don't exist
        for script in self.distribution.scripts:
            if os.path.islink(script):
                target = os.readlink(script)
                newlink = os.path.join(self.install_dir,
                                       os.path.basename(script))
                if not os.path.exists(newlink):
                    os.symlink(target, newlink)


class build_py_with_git_version(build_py):
    '''Like build_py, but also hardcoding the version in __init__.__version__
       so it's consistent even outside of the source tree'''

    def build_module(self, module, module_file, package):
        build_py.build_module(self, module, module_file, package)
        print module, module_file, package
        if module == '__init__' and '.' not in package:
            version_line = "__version__ = '{0}'\n".format(__version__)
            old_init_name = self.get_module_outfile(self.build_lib, (package,),
                                                    module)
            new_init_name = old_init_name + '.new'
            with open(new_init_name, 'w') as new_init:
                with open(old_init_name) as old_init:
                    for line in old_init:
                        if line.startswith('__version__ ='):
                            new_init.write(version_line)
                        else:
                            new_init.write(line)
                new_init.flush()
            os.rename(new_init_name, old_init_name)


class sdist_with_git_version(sdist):
    '''Like sdist, but also hardcoding the version in __init__.__version__ so
       it's consistent even outside of the source tree'''

    def make_release_tree(self, base_dir, files):
        sdist.make_release_tree(self, base_dir, files)
        version_line = "__version__ = '{0}'\n".format(__version__)
        old_init_name = os.path.join(base_dir, 'euca2ools/__init__.py')
        new_init_name = old_init_name + '.new'
        with open(new_init_name, 'w') as new_init:
            with open(old_init_name) as old_init:
                for line in old_init:
                    if line.startswith('__version__ ='):
                        new_init.write(version_line)
                    else:
                        new_init.write(line)
            new_init.flush()
        os.rename(new_init_name, old_init_name)


setup(name="euca2ools",
      version=__version__,
      description="Eucalyptus Command Line Tools",
      long_description="Eucalyptus Command Line Tools",
      author="Eucalyptus Systems, Inc.",
      author_email="support@eucalyptus.com",
      url="http://www.eucalyptus.com",
      scripts=sum((glob.glob('bin/euare-*'),
                   glob.glob('bin/euca-*'),
                   glob.glob('bin/euform-*'),
                   glob.glob('bin/euimage-*'),
                   glob.glob('bin/eulb-*'),
                   glob.glob('bin/euscale-*'),
                   glob.glob('bin/euwatch-*')),
                  []),
      data_files=[('share/man/man1', glob.glob('man/*.1')),
                  ('share/man/man5', glob.glob('man/*.5')),
                  ('share/man/man7', glob.glob('man/*.7'))],
      packages=find_packages(),
      install_requires=REQUIREMENTS,
      license='BSD (Simplified)',
      platforms='Posix; MacOS X',
      classifiers=['Development Status :: 5 - Production/Stable',
                   'Intended Audience :: Developers',
                   'Intended Audience :: System Administrators',
                   'License :: OSI Approved :: Simplified BSD License',
                   'Operating System :: OS Independent',
                   'Programming Language :: Python',
                   'Programming Language :: Python :: 2',
                   'Programming Language :: Python :: 2.6',
                   'Programming Language :: Python :: 2.7',
                   'Topic :: Internet'],
      cmdclass={'build_py': build_py_with_git_version,
                'build_scripts': build_scripts_except_symlinks,
                'install_scripts': install_scripts_and_symlinks,
                'sdist': sdist_with_git_version})



# Copyright 2013-2014 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from requestbuilder import Arg
from requestbuilder.mixins import TabifyingMixin

from euca2ools.commands.ec2 import EC2Request


class DescribeInstanceTypes(EC2Request, TabifyingMixin):
    DESCRIPTION = '[Eucalyptus only] Show information about instance types'
    ARGS = [Arg('InstanceType', metavar='INSTANCETYPE', nargs='*',
                help='limit results to specific instance types'),
            Arg('--by-zone', dest='by_zone', action='store_true',
                route_to=None,
                help='show info for each availability zone separately'),
            Arg('--show-capacity', dest='Availability', action='store_true',
                help='show info about instance capacity')]
    LIST_TAGS = ['instanceTypeDetails', 'availability']

    def configure(self):
        EC2Request.configure(self)
        if self.args.get('by_zone', False):
            self.params['Availability'] = True

    def print_result(self, result):
        vmtype_names = []  # Use a list since py2.6 lacks OrderedDict
        vmtypes = {}  # vmtype -> info and total capacity
        zones = {}  # zone -> vmtype -> info and zone capacity
        for vmtype in result.get('instanceTypeDetails', []):
            vmtype_names.append(vmtype['name'])
            vmtypes[vmtype['name']] = {'cpu': vmtype.get('cpu'),
                                       'memory': vmtype.get('memory'),
                                       'disk': vmtype.get('disk'),
                                       'available': 0,
                                       'max': 0}
            if self.params.get('Availability', False):
                for zone in vmtype.get('availability', []):
                    available = int(zone.get('available', 0))
                    max_ = int(zone.get('max', 0))
                    vmtypes[vmtype['name']]['available'] += available
                    vmtypes[vmtype['name']]['max'] += max_
                    zones.setdefault(zone['zoneName'], {})
                    zones[zone['zoneName']][vmtype['name']] = {
                        'cpu': vmtype.get('cpu'),
                        'memory': vmtype.get('memory'),
                        'disk': vmtype.get('disk'),
                        'available': available,
                        'max': max_}

        if self.args.get('by_zone'):
            for zone, zone_vmtypes in sorted(zones.iteritems()):
                print self.tabify(('AVAILABILITYZONE', zone))
                self._print_vmtypes(zone_vmtypes, vmtype_names)
                print
        else:
            self._print_vmtypes(vmtypes, vmtype_names)

    def _print_vmtypes(self, vmtypes, vmtype_names):
        # Fields and column headers
        fields = {'name': 'Name',
                  'cpu': 'CPUs',
                  'memory': 'Memory (MiB)',
                  'disk': 'Disk (GiB)',
                  'used': 'Used',
                  'total': 'Total',
                  'used_pct': 'Used %'}
        field_lengths = dict((field, len(header)) for field, header
                             in fields.iteritems())
        vmtype_infos = []
        for vmtype_name in vmtype_names:
            total = int(vmtypes[vmtype_name].get('max', 0))
            used = total - int(vmtypes[vmtype_name].get('available', 0))
            if total != 0:
                used_pct = '{0:.0%}'.format(float(used) / float(total))
            else:
                used_pct = ''
            vmtype_info = {'name': vmtype_name,
                           'cpu': vmtypes[vmtype_name].get('cpu'),
                           'memory': vmtypes[vmtype_name].get('memory'),
                           'disk': vmtypes[vmtype_name].get('disk'),
                           'used': used,
                           'total': total,
                           'used_pct': used_pct}
            vmtype_infos.append(vmtype_info)
            for field in fields:
                if len(str(vmtype_info[field])) > field_lengths[field]:
                    field_lengths[field] = len(str(vmtype_info[field]))
        type_template = ('{{name:<{name}}}  {{cpu:>{cpu}}}  '
                         '{{memory:>{memory}}}  {{disk:>{disk}}}')
        if self.args.get('Availability', False):
            type_template += ('  {{used:>{used}}} / {{total:>{total}}}  '
                              '{{used_pct:>{used_pct}}}')
        type_template = type_template.format(**field_lengths)

        print 'INSTANCETYPE\t', type_template.format(**fields)
        for vmtype_info in vmtype_infos:
            print 'INSTANCETYPE\t', type_template.format(**vmtype_info)



# Copyright 2013 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from argparse import ArgumentTypeError


def autoscaling_filter_def(filter_str):
    filter_dict = {}
    pieces = filter_str.split(',')
    for piece in pieces:
        piece = piece.strip()
        if '=' not in piece:
            raise ArgumentTypeError(
                "invalid filter: each segment of '{0}' must have format "
                "KEY=VALUE".format(piece))
        key, val = piece.split('=', 1)
        filter_dict.setdefault(key.strip(), [])
        filter_dict[key.strip()].append(val.strip())
    filter_list = []
    for key, values in filter_dict.iteritems():
        filter_list.append({'Name': key, 'Values': values})
    return filter_list


def autoscaling_tag_def(tag_str):
    tag_dict = {}
    pieces = tag_str.split(',')
    for piece in pieces:
        piece = piece.strip()
        if '=' not in piece:
            raise ArgumentTypeError(
                "invalid tag definition: each segment of '{0}' must have "
                "format KEY=VALUE".format(piece))
        key, val = piece.split('=', 1)
        if key == 'k':
            tag_dict['Key'] = val
        elif key == 'id':
            tag_dict['ResourceId'] = val
        elif key == 't':
            tag_dict['ResourceType'] = val
        elif key == 'v':
            tag_dict['Value'] = val
        elif key == 'p':
            if val.lower() in ('true', 'false'):
                tag_dict['PropagateAtLaunch'] = val.lower()
            else:
                raise ArgumentTypeError(
                    "value for to 'p=' must be 'true' or 'false'")
        else:
            raise ArgumentTypeError(
                "unrecognized tag segment '{0}'".format(piece))
    if not tag_dict.get('Key'):
        raise ArgumentTypeError(
            "tag '{0}' must contain a 'k=' segment with a non-empty value")
    return tag_dict



# Copyright 2013 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from euca2ools.commands.monitoring import CloudWatchRequest
from requestbuilder import Arg
from requestbuilder.mixins import TabifyingMixin
from requestbuilder.response import PaginatedResponse


class DescribeAlarmHistory(CloudWatchRequest, TabifyingMixin):
    DESCRIPTION = 'Retrieve history for one alarm or all alarms'
    ARGS = [Arg('AlarmName', metavar='ALARM', nargs='?',
                help='limit results to a specific alarm'),
            Arg('--end-date', dest='EndDate', metavar='DATE',
                help='limit results to history before a given point in time'),
            Arg('--history-item-type', dest='HistoryItemType',
                choices=('Action', 'ConfigurationUpdate', 'StateUpdate'),
                help='limit results to specific history item types'),
            Arg('--show-long', action='store_true', route_to=None,
                help='show detailed event data as machine-readable JSON'),
            Arg('--start-date', dest='StartDate', metavar='DATE',
                help='limit results to history after a given point in time')]
    LIST_TAGS = ['AlarmHistoryItems']

    def main(self):
        return PaginatedResponse(self, (None,), ('AlarmHistoryItems',))

    def prepare_for_page(self, page):
        self.params['NextToken'] = page

    def get_next_page(self, response):
        return response.get('NextToken') or None

    def print_result(self, result):
        for item in result.get('AlarmHistoryItems', []):
            bits = [item.get('AlarmName'), item.get('Timestamp'),
                    item.get('HistoryItemType'), item.get('HistorySummary')]
            if self.args['show_long']:
                bits.append(item.get('HistoryData'))
            print self.tabify(bits)
